{#- base template for communities (as container for social apps) #}
{% extends "base.html" %}
{% from "macros/box.html" import m_box_forum %}
{%- set steps =[_("Add Members"),_("Existing and new members"),_("Create Accounts")] %}

{% block header %}
  <h1 class="community-header">
    <img src="{{ community_image_url(g.community, s=40) }}"/>
    <span style="color:black;">{{ g.community.name }}</span>
    {% if threads %} -
      <span class="forum-nb-conversation">{{ threads|length }} Conversation(s)</span>
    {% endif %}
  </h1>
  {%- set tabs = actions.for_category('communities:tabs') %}

  <ul class="nav nav-tabs community-tabs" id="community-tabs">
    {%- for tab in tabs %}
      <li {% if tab.is_current() %}class="active"{% endif %}>{{ tab.render() }}</li>
    {%- endfor %}
  </ul>
{% endblock %}

{% block main %}
  <div class="row">
    <div class="col-md-9">
      {%- block content %}
      {%- endblock %}
    </div>

    <div class="col-md-3" id="sidebar">
      {%- block sidebar %}
      {%- endblock %}
    </div>
  </div>


  {%- block forumcontent %}
  {%- endblock %}

  {%- block memberscontent %}
  {%- endblock %}

  {%- block modals %}
  {% endblock %}

{% endblock %}
