// Community header
table.community-list tr {
  td {
    vertical-align: middle;

    &.logo {
      width: 65px !important;
      height: 65px !important;

      img {
        border-radius: 10%;
      }
    }

    a {
      text-decoration: none;
      color: #555; /* TODO: global font color */
    }

    .profile {
      margin: 10px 0 10px 10px;
    }

    .name {
      margin: 2px 0;
      font-size: 1.5em;
      font-weight: bold;
    }

    .description {
      font-style: italic;
    }
  }

  &:last-child td {
    border-bottom: 1px solid #ddd;
  }
}

.members-table {
  > thead > tr > th {
    text-align: center;
  }

  > tbody > tr > td:nth-child(n+2) {
    text-align: center;
  }
}

.members-download-btn {
  float: right;
  position: relative;
  top: 39px;
}

.members-title {
  position: relative;
  font-weight: normal;
  color: silver;
  font-family: sans-serif;
  top: 7px;
  margin-bottom: 21px;
}

.members-data {
  position: relative;
  top: 7.5px;
  text-align: center;
}

